﻿#region Details
// Class  - BackgroundScroller 
// Author - Chris 240115
// Description:
// BackgroundScroller Class
// Creates scrolling background by offsetting the texture
// Functionality to access speed
// *Must be attached directly to a gameobject containing a texture*
#endregion

#region Includes
using UnityEngine;
using System.Collections;
#endregion

// BackgroundScroller //
public class BackgroundScroller : MonoBehaviour {
	// Unity Editor Variables //
	[ SerializeField ] private float mScrollSpeed = 0.0f;
	[ SerializeField ] private float mMaxSpeed    = 0.0f;

	// Private Variables //
	private Vector2 mSavedOffset;
	private bool    mSlowingDown = false;
	private bool    mSpeedingUp  = true;
	private float   mTime        = 0.0f;
	private float   mScrollTime  = 0.0f;

	// Start                              //
	// Save initial texture offset values //
	void Start() {
		mSavedOffset = renderer.sharedMaterial.GetTextureOffset( "_MainTex" );
	}

	// Update                                                                     //
	// Scroll in x (horizontal) by incrementing y texture offset value from 0 - 1 // 
	void Update() {
		// Only when playing
		if( GameController.Instance.GetState() == GameController.GameState.PLAYING ) {
			// If slowing down
			if( mSlowingDown ) {
				mTime += Time.deltaTime * 0.6f;
				mScrollSpeed = Mathf.Lerp( mMaxSpeed, 0.0f, mTime );

				// Once target reached - switch to speeding up
				if( mScrollSpeed == 0.0f ) {
					mSlowingDown = false;
					mSpeedingUp  = true;
					mTime = 0.0f;
				}
			// If speeding up
			} else if ( mSpeedingUp ) {
				mTime += Time.deltaTime * 0.3f;
				mScrollSpeed = Mathf.Lerp( 0.0f, mMaxSpeed, mTime );

				// Once target reached - stop speeding up
				if( mScrollSpeed == mMaxSpeed ) {
					mSpeedingUp = false;
					mTime = 0.0f;
				}
			}

			// Increase scroll time by delta time - which is scaled by the desired scroll speed
			mScrollTime += ( Time.deltaTime * mScrollSpeed );

			// Use Time, limit to max of 1 
			float x = Mathf.Repeat( mScrollTime /*Time.time * mScrollSpeed*/, 1 );

			// Assign value to Vector2
			Vector2 offset = new Vector2( x, mSavedOffset.y );

			// Access renderer
			renderer.sharedMaterial.SetTextureOffset( "_MainTex", offset );

			// Add score
			GameController.Instance.IncreaseScore( ( int )( 5.0f * mScrollSpeed ) );
		}
	}

	// OnDisable                                //
	// Return texture offsets to orignal values //
	void OnDisable() {
		renderer.sharedMaterial.SetTextureOffset( "_MainTex", mSavedOffset );
	}

	// SlowDown //
	// Triggers a quick stop, followed by slower acceleration back to max speed //
	public void SlowDown() {
		if( !mSlowingDown && !mSpeedingUp ) {
			mSlowingDown = true;
			mTime = 0.0f;
		}
	}

	// GetSpeed           //
	// Gets current speed //
	public float GetSpeed() {
		return mScrollSpeed;
	}
}