﻿#region Details
// Class  - LevelController 
// Author - Chris 230115
// Description:
// Level Controller Class
// Contains all objects related to the level implementation
// - Background tiles
// - Platforms
// - Obstacles
// Controls correct spawning and removal of those objects
#endregion

#region Includes
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
#endregion

// LevelController //
public class LevelController : MonoBehaviour {
	// Unity Editor Variables //
	[ SerializeField ] private GameObject         mSmallObstacle    = null;
	[ SerializeField ] private GameObject         mRedBinObstacle   = null;
	[ SerializeField ] private GameObject         mBlueCatObstacle  = null;
	[ SerializeField ] private GameObject         mCatOnBinObstacle = null;
	[ SerializeField ] private BackgroundScroller mGround           = null;
	[ SerializeField ] private BackgroundScroller mBackground       = null;

	[ SerializeField ] private Player             mBluedogRef       = null;
	[ SerializeField ] private Player             mReddogRef        = null;

	// Private Variables //
	private List< GameObject > mObstacles;
	private float mTimer     = 0.0f;
	private float mSpawnTime = 0.0f;
	
	// Start                       //	
	// Use this for initialization //
	void Start () {
		// Initialise objects list
		mObstacles = new List< GameObject >();
	}

	// Update                                                            //
	// Moves, checks for collision flags and removes offscreen obstacles //
	void Update () {
		// Only when playing
		if( GameController.Instance.GetState() == GameController.GameState.PLAYING ) {
			// Spawn control
			if( mTimer > mSpawnTime ) {
				SpawnObstacle();
				mSpawnTime = UnityEngine.Random.Range ( 0, 3 ) + 5;
				mTimer = 0.0f;
			}

			for( int i = 0; i < mObstacles.Count; i++ ) {
				// Get obstacle component of this gameobject
				Obstacle temp = mObstacles[ i ].GetComponent< Obstacle >();

				// Move each obstacle
				temp.Move( mGround.GetSpeed() * 0.25f );

				// Check if has collided
				if( temp.HasBeenHit() ) {
					// Trigger the slow down
					mGround.SlowDown();
					mBackground.SlowDown();

					// Reset the flag
					temp.HasBeenTriggered();
				}

				// Check if leaving the scene
				if( RemoveOutOfBounds( i ) ) {
					break;
				}

				// Obstacle removal conditions //
				// Remove blue obstacle if bluedog is leading
				if(  mBluedogRef.GetPosition() == Player.Position.LEADING ) {
					if( temp.GetObstacleType() == Obstacle.ObstacleType.BLUE ) {
						if( ( temp.transform.position.x - mBluedogRef.transform.position.x ) < 3.0f ) {
							ThrowObstacle( i );
						}
					}
				// Remove red obstacle if reddog is leading
				} else if ( mReddogRef.GetPosition() == Player.Position.LEADING ) {
					if( temp.GetObstacleType() == Obstacle.ObstacleType.RED ) {
						if( ( temp.transform.position.x - mReddogRef.transform.position.x ) < 3.0f ) {
							ThrowObstacle( i );
						}
					}
				}  
				// Remove purple obstacle is dogs are switching position
				if( mBluedogRef.IsSwitching() || mReddogRef.IsSwitching() ) {
					if( temp.GetObstacleType() == Obstacle.ObstacleType.PURPLE ) {
						if( ( ( temp.transform.position.x - mReddogRef.transform.position.x ) < 3.0f ) || 
						    ( ( temp.transform.position.x - mBluedogRef.transform.position.x ) < 3.0f ) ) {
							ThrowObstacle( i );
						}
					}
				}
			}

			// Increment timer
			mTimer += Time.deltaTime;

			// Timer reset
			if( mTimer > 10000.0f ) {
				mTimer = 0.0f;
			}
		}
	}

	// RemoveOutOfBounds               //
	// Removes out of bounds obstacles //
	bool RemoveOutOfBounds( int index ) {
		// Four boundaries - remove and return true
		if( mObstacles[ index ].transform.position.x < -12.0f ) {
			RemoveObstacle( index );
			return true;
		} else if( mObstacles[ index ].transform.position.x > 20.0f ) {
			RemoveObstacle( index );
			return true;
		} else if( mObstacles[ index ].transform.position.y < -20.0f ) {
			RemoveObstacle( index );
			return true;
		} else if( mObstacles[ index ].transform.position.y > 20.0f ) {
			RemoveObstacle( index );
			return true;
		}

		// Nothing removed return false
		return false;
	}

	// ThrowObstacle                     //
	// Use force to throw obstacle clear //
	void ThrowObstacle( int index ) {
		// Remove collider
		Destroy( mObstacles[ index ].GetComponent< PolygonCollider2D >() );

		// Launch into orbit
		mObstacles[ index ].GetComponent< Rigidbody2D >().AddForce( new Vector2( 0.0f, 30.0f ) );

		// Add score
		GameController.Instance.IncreaseScore( 100 );
	}

	// RemoveObstacle                                      //
	// Removes Gameobject from both the list and the scene //
	void RemoveObstacle( int index ) {
		GameObject toBeRemoved = mObstacles[ index ];
		mObstacles.Remove( toBeRemoved );
		Destroy( toBeRemoved );
	}

	// SpawnObstacle            //
	// Spawns a random obstacle //
	void SpawnObstacle() {
		int random = UnityEngine.Random.Range( 0, 5 );
		if( ( random == 0 ) || ( random == 1 ) ) {
			GameObject smallObstacle = ( GameObject )Instantiate( mSmallObstacle, new Vector3( 12.0f, -3.75f, 0.0f ), Quaternion.identity ); 
			mObstacles.Add( smallObstacle );
		} else if( random == 2 ) {
			GameObject redBinObstacle = ( GameObject )Instantiate( mRedBinObstacle, new Vector3( 12.0f, -2.75f, 0.0f ), Quaternion.identity );
			mObstacles.Add( redBinObstacle );
		} else if( random == 3 ) {
			GameObject catObstacle = ( GameObject )Instantiate( mBlueCatObstacle, new Vector3( 12.0f, -3.5f, 0.0f ), Quaternion.identity );
			mObstacles.Add( catObstacle );
		} else if( random == 4 ) {
			GameObject catOnBinObstacle = ( GameObject )Instantiate( mCatOnBinObstacle, new Vector3( 12.0f, -2.0f, 0.0f ), Quaternion.identity );
			mObstacles.Add( catOnBinObstacle );
		}
	}
}