﻿#region Details
// Class  - Player 
// Author - Chris 230115
// Description:
// Player Class
// One of two player controlled objects
// Can jump and switch into the lead position
#endregion

#region Includes
using UnityEngine;
using System.Collections;
#endregion

// Player //
public class Player : MonoBehaviour {
	// Position Variables //
	public enum Position { LEADING, FOLLOWING };
	private     Position mCurrentPosition;
	private     Position mTargetPosition;
	
	// Unity Editor Variables //
	[ SerializeField ] private Vector2   mJumpVector   = new Vector2( 0.0f, 200.0f );
	[ SerializeField ] private Transform mGroundCheck  = null;
	[ SerializeField ] private LayerMask mWhatIsGround = -1;

	// Private Variables //
	private Rigidbody2D mRigidBody2D   = null;
	private bool        mGrounded      = false;
	private bool        mJumping       = false;
	private Vector2     mLeadVector    = new Vector2( -3.0f, 0.0f );
	private Vector2     mFollowVector  = new Vector2( -7.0f, 0.0f );
	private float       mTime          = 0.0f;
	private bool        mSwitching     = false;

	private Animator mAnimator;

	// Start                       //
	// Use this for initialization //
	void Start () {
		// Get Rigidbody2D component
		mRigidBody2D = GetComponent< Rigidbody2D >();

		// Get animator component
		mAnimator = GetComponent< Animator >();
	}

	// FixedUpdate    //
	// Physics update //
	void FixedUpdate () {
		// Check if grounded
		mGrounded = Physics2D.OverlapPoint( mGroundCheck.position, mWhatIsGround );

		// Only jump if grounded
		if( mGrounded ) {
			// Jump
			if( mJumping ) {
				mRigidBody2D.AddForce( mJumpVector );
			}
		}

		// Setting jump animation bool
		if( mGrounded ) {
			mAnimator.SetBool( "isJumping", false );
		} else {
			mAnimator.SetBool( "isJumping", true );
		}

		// Reset jumping flag
		mJumping = false;
	}

	// Update //
	// Checks if player has been assigned a new position - either leading or following //
	// Then lerp's to the correct position                                             //
	void Update() {
		// If target position has been changed
		if( ( mCurrentPosition != mTargetPosition ) && mSwitching ) {
			// Lerp Vector2
			Vector2 newPosition = Vector2.zero;

			// Start increasing time
			mTime += Time.deltaTime * 0.7f;

			// Either moving into lead or falling back to follow
			switch( mTargetPosition ) {
				case Position.FOLLOWING: {
					newPosition = Vector2.Lerp( mLeadVector, mFollowVector, mTime );
					transform.position = new Vector3( newPosition.x, transform.position.y, newPosition.y );
					if( ( transform.position.x == mFollowVector.x ) && ( transform.position.z == mFollowVector.y ) ) {
						mCurrentPosition = mTargetPosition;
					}
					break;
				}
				case Position.LEADING: {
					newPosition = Vector2.Lerp( mFollowVector, mLeadVector, mTime );
					transform.position = new Vector3( newPosition.x, transform.position.y, newPosition.y );
					if( ( transform.position.x == mLeadVector.x ) && ( transform.position.z == mLeadVector.y ) ) {
						mCurrentPosition = mTargetPosition;
					}
					break;
				}
			}

		} else {
			mTime = 0.0f;
			mSwitching = false;
		}
	}

	// SetPosition                                 //
	// Function to set the player objects position //
	public void SetPosition( Position newPosition ) {
		// Set incoming position state
		mCurrentPosition = newPosition;
		mTargetPosition  = newPosition;

		// Move player object ot corret starting position
		if( mTargetPosition == Position.LEADING ) {
			transform.position = new Vector3( mLeadVector.x, transform.position.y, mLeadVector.y );
		}
		if( mTargetPosition == Position.FOLLOWING ) {
			transform.position = new Vector3( mFollowVector.x, transform.position.y, mFollowVector.y );
		}
	}

	// GetPosition                        //
	// Returns the players position state //
	public Position GetPosition() {
		return mCurrentPosition;
	}

	// IsSwitching                                   //
	// Returns true if players are swapping position //
	public bool IsSwitching() {
		return mSwitching;
	}

	// Jump  //
	// Jump! //
	public void Jump() {
		mJumping = true;
	}

	// TakeLead                                                   // 
	// Commands this player object to take the 'leading' position //
	public void TakeLead() {
		if( !mSwitching ) {
			if( mCurrentPosition == Position.LEADING ) {
				mTargetPosition = Position.FOLLOWING;
			} else if( mCurrentPosition == Position.FOLLOWING ) {
				mTargetPosition = Position.LEADING;
			}
			mSwitching = true;
		}
	}
	
	// IsDazed              //
	// Sets isDazed trigger //
	public void IsDazed() {
		mAnimator.SetTrigger( "isDazed" );
	}
}